#!/bin/bash
# -------
# File:        llxcfg-common.sh
# Description: common functions for llxcfg system
#              May 2007, Luis Garcia <garcia_luigis@gva.es>
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along 
# with this program; if not, write to the Free Software Foundation, Inc., 
# 51 Franklin St, Fifth Floor, Boston MA 02110-1301 USA
# --------

PATH="/usr/sbin:/usr/bin:/sbin:/bin"

# add-ons
# ------- #
LLXEXT_BASE_DIR="/usr/share/llxcfg-addons"
LLXEXT_SOURCE="$LLXEXT_BASE_DIR/common.d"
LLXEXT_DEVEL="$LLXEXT_BASE_DIR/devel"
#LLXEXT_TYPES="source devel"
LLXEXT_DIRS="$LLXEXT_SOURCE $LLXEXT_DEVEL"

llxext_listfiles(){
   [ "$1" -a -d "$1" ] || return 0
   find "$1" -maxdepth 1 -mindepth 1 -xtype f |sort
   return 0
}

for f in $(llxext_listfiles "$LLXEXT_SOURCE") ; do
   . "$f"
done

# llxcfg-scripts #
# -------------- #

LLXSCR_DIR="/etc/llxcfg"
LLXSCR_DENYFILE="llxcfg-scr.deny"
LLXSCR_STOP_ON_ERROR=""

### add "custom types" here (if any)
LLXSCR_USER="user"
LLXSCR_SESSION="session"
LLXSCR_TYPES="${LLXSCR_TYPES} ${LLXSCR_USER} ${LLXSCR_SESSION}"

### BEGIN LLX AUTOMAGIC SCRIPTS VARIABLES
### END LLX AUTOMAGIC SCRIPTS VARIABLES

# ERRORS
RC_SCR_UNKNOWTYPE=5
RC_SCR_BADSCRNAME=10
RC_SCR_NODIR=15
RC_SCR_NOTENABLED=20
RC_SCR_RUNFAILED=25   

# FUNCTIONS

llxscr_typetest(){
   echo " ${LLXSCR_TYPES} " |grep -q " ${1} " || return ${RC_SCR_UNKNOWTYPE}
   return 0
}

llxscr_dir(){
   echo "${LLXSCR_DIR}/${1}-scripts"
}

llxscr_endir(){
   echo "${LLXSCR_DIR}/${1}-scripts.d"
}

llxscr_denydir(){
   echo "${LLXSCR_DIR}/${1}-scripts.deny"
}

llxscr_file(){
   echo "$(llxscr_dir "$1")/$2"
}

llxscr_relfile(){
   echo "../${1}-scripts/$2"
}

llxscr_enfile(){
   NUM_ORDER=$(printf "%03i" $(echo "$3"|sed -e "s%^0*%%"))
   DEFAULT_ORDER=500
   if [ $NUM_ORDER -gt 999 -o $NUM_ORDER -le 0 ] ; then
      NUM_ORDER=${DEFAULT_ORDER}
   fi 
   echo "$(llxscr_endir "$1")/${NUM_ORDER}-$2"
}

llxscr_scrtest(){
   [ -x "$(llxscr_file "$1" "$2")" ] || return $RC_SCR_BADSCRNAME
   return 0
}

llxscr_enscrtest(){
   [ "$(find "$(llxscr_endir "$1")" -lname "$(llxscr_relfile "$1" "$2")" 2>/dev/null)" ] && return 0
   return $RC_SCR_NOTENABLED
}

llxscr_enable(){
   SCR_TYPE="$1"
   SCR_NAME="$2"
   SCR_NUM="$3"
   rc=0
   llxscr_typetest "$SCR_TYPE" || return $?
   llxscr_scrtest "$SCR_TYPE" "$SCR_NAME" || return $?
   llxscr_enscrtest "$SCR_TYPE" "$SCR_NAME" && return 0
   ENABLED_DIR="$(llxscr_endir "${SCR_TYPE}")"
   SCR_FILE="$(llxscr_relfile "${SCR_TYPE}" "$SCR_NAME")"
   ENSCR_FILE="$(llxscr_enfile "${SCR_TYPE}" "$SCR_NAME" "$SCR_NUM")"
   [ -d "${ENABLED_DIR}" ] || return RC_SCR_NODIR
   ln -s "${SCR_FILE}" "${ENSCR_FILE}"
   return 0
}

llxscr_testdeny(){
   SCR_TYPE="$1"
   SCR_NAME="$2"
   DENIED_SCRIPTS_VARNAME="$LLX_${SCR_TYPE}SCRIPTS_DENY"
   DENIED_SCRIPTS_LIST="${!DENIED_SCRIPTS_VARNAME}"
   if echo " $DENIED_SCRIPTS_LIST " |grep -q " ${SCR_NAME} " ; then
      return 0
   fi
   DENY_DIR="$(llxscr_denydir "${SCR_TYPE}")"
   if rgrep -q "^${SCR_NAME}$" "${DENY_DIR}" 2>/dev/null ; then
      return 0
   fi   
   return 1
}

llxscr_disable(){
   SCR_TYPE="$1"
   SCR_NAME="$2"
   rc=0
   llxscr_typetest "$SCR_TYPE" || return $?
   llxscr_enscrtest "$SCR_TYPE" "$SCR_NAME" || return $?
   find "$(llxscr_endir "$SCR_TYPE")" -lname "$(llxscr_relfile "$SCR_TYPE" "$SCR_NAME")" 2>/dev/null |while read ENSCR_FILE ; do
      rm -f "${ENSCR_FILE}"
   done
   return 0
}


llxscr_deny(){
   SCR_TYPE="$1"
   SCR_NAME="$2"
   rc=0
   DENY_DIR="$(llxscr_denydir "${SCR_TYPE}")"
   [ -d "${DENY_DIR}" ] || mkdir -p "${DENY_DIR}"
   DENY_FILE="${DENY_DIR}/${LLXSCR_DENYFILE}"
   [ -f "${DENY_FILE}" ] || touch "${DENY_FILE}"
   grep -q "^${SCR_NAME}$" "${DENY_FILE}" && return 0
   echo "${SCR_NAME}" |llxcfg-install - "${DENY_FILE}" || return $?
   return 0
}

llxscr_allow(){
   SCR_TYPE="$1"
   SCR_NAME="$2"
   rc=0
   DENY_DIR="$(llxscr_denydir "${SCR_TYPE}")"
   [ -d "${DENY_DIR}" ] || return 0
   DENY_FILE="${DENY_DIR}/${LLXSCR_DENYFILE}"
   [ -f "${DENY_FILE}" ] || return 0
   grep -q "^${SCR_NAME}$" "${DENY_FILE}" || return 0
   grep -v "^${SCR_NAME}$" "${DENY_FILE}" |llxcfg-install - "${DENY_FILE}" || return $?
   return 0
}

llxscr_log(){
   local scr_rc LOG_ACTION
   scr_rc=0
   SCR_TYPE="$1"
   shift
   SCR_NAME="$1"
   shift
   LOG_FILE="${LLXSCR_LOG_DIR}/${SCR_TYPE}.log"
   if [ "$LLXSCR_LOG_DIR" ] ; then
      LINE="$(date "+%F%t%T%t$SCR_NAME%tCOMMAND:")"
      echo "${LINE}${@}" >> "$LOG_FILE"
   fi
   "$@" || scr_rc=$?
   if [ "$LLXSCR_LOG_DIR" ] ; then
      date "+%F%t%T%t$SCR_NAME%tRESULT:$scr_rc" >> "$LOG_FILE"
   fi
   return $scr_rc
}

llxscr_run(){
   local scr_rc
   rc=0
   SCR_TYPE="$1"
   shift
   for ENSCR_FILE in $(find "$(llxscr_endir "$SCR_TYPE")" -xtype f |sort) ; do
      if [ -x "$ENSCR_FILE" ] ; then
         SCR_NAME="$(basename "$(readlink -f "$ENSCR_FILE")")"
         if ! llxscr_testdeny "$SCR_TYPE" "$SCR_NAME" ; then
            scr_rc=0
            llxscr_log "$SCR_TYPE" "$SCR_NAME" "${ENSCR_FILE}" "$@" || scr_rc=$?
            if [ $scr_rc -ne 0 ] ; then
               if [ "$LLXSCR_STOP_ON_ERROR" ] ; then
                  rc=$scr_rc
                  break
               fi
               rc=${RC_SCR_RUNFAILED}
            fi
         fi  
      fi
   done
   return $rc
}

# llxcfg-cpkg    #
# -------------- #

# ---------
# variables
# ---------
CPKG_BACKEND="/usr/share/lliurex/llxcfg-backend/llxcfg-cpkg-backend"

CPKG_DIR="/var/lib/llxcfg/cpkg"
CPKG_INFODIRNAME="info"
CPKG_INFO="${CPKG_DIR}/${CPKG_INFODIRNAME}"
CPKG_DENIED="${CPKG_DIR}/deny"
CPKG_TMP="${CPKG_DIR}/tmp"
CPKG_ENDIRNAME="enabled"
CPKG_ENABLED="${CPKG_DIR}/${CPKG_ENDIRNAME}"
CPKG_FIRST_BACKUP="${CPKG_DIR}/backup/first"
CPKG_LAST_BACKUP="${CPKG_DIR}/backup/last"
CPKG_DIRS="${CPKG_INFO}"
CPKG_SCRNAMES="preconfig postconfig upgrade"
CPKG_CP_PREFIX="CP_"
CPKG_RM_PREFIX="RM_"
CPKG_CONTROL_FILE="cpkg.control"
CPKG_CONF_FILE="sysfiles.list"
CPKG_USER_FILE="userfiles.list"
CPKG_SERVICES_FILE="services.list"
CPKG_INSTALL_FILE="install.list"
CPKG_PKGS_FILE="packages.list"
CPKG_ALIAS_FILE="alias.list"
CPKG_TEMPLATES_FILE="templates.list"
CPKG_ORIGINAL="${CPKG_DIR}/packages"
CPKG_EXCLUDE_LIST="$CPKG_DIR/lists/cpkg-exclude"
CPKG_EXCLUDE_CONFDIR="cpkg/lists/cpkg-exclude"
CPKG_PREOVERRIDES_DIR="/etc/llxcfg/cpkg/pre-overrides"
CPKG_POSTOVERRIDES_DIR="/etc/llxcfg/cpkg/post-overrides"
CPKG_UPDATE_DIR="/usr/share/llxcfg/cpkg-update"
DEFAULT_ORDER=500
DEFAULT_DENYFILE="${CPKG_DENIED}/llxcfg-cpkg.deny"
DEFAULT_SOURCE="skel"
DEFAULT_TEMPLATE="templates"
DEFAULT_TARGET="/"

PKG_MSG_LEN=35

RC_NOTFOUND=1
RC_DISABLED=5

# -----------
# functions
# -----------

cpkg_dir(){
   echo "${CPKG_INFO}/$1"
}


cpkg_reldir(){
   echo "../${CPKG_INFODIRNAME}/$1"
}

cpkg_endir(){
   pkg="$1"
   NUM_ORDER=$(printf "%03i" $(echo "$2"|sed -e "s%^0*%%"))
   if [ $NUM_ORDER -gt 999 -o $NUM_ORDER -le 0 ] ; then
      NUM_ORDER=${DEFAULT_ORDER}
   fi
   echo "${CPKG_ENABLED}/${NUM_ORDER}-${1}"
}


# llxcfg-vars    #
# -------------- #
LLXVARS_DIR1="/usr/share/llxcfg/vars"
LLXVARS_DIR2="/var/lib/llxcfg/vars"
LLXVARS_DIR3="/etc/llxcfg/vars"
LLXVARS_DIRS="$LLXVARS_DIR1 $LLXVARS_DIR2 $LLXVARS_DIR3"


# llxcfg-vpkg    #
# -------------- #

# ---------
# variables
# ---------
VPKG_DIR="/var/lib/llxcfg/vpkg"
VPKG_INFODIRNAME="info"
VPKG_INFO="${VPKG_DIR}/${VPKG_INFODIRNAME}"
VPKG_ENDIRNAME="enabled"
VPKG_ENABLED="${VPKG_DIR}/${VPKG_ENDIRNAME}"
VPKG_OVERRIDES="${VPKG_DIR}/overrides"
VPKG_STATIC="default"
VPKG_DYNAMIC="generate"
VPKG_VALIDATE="postgenerate"
VPKG_OVERRIDE="override"
VPK_VARS="vars"
VPKG_STATIC_FILE="/etc/llxcfg/vars.DEFAULT"
VPKG_OVERRIDE_FILE="/etc/llxcfg/vars.OVERRIDE"
VPKG_DYNAMIC_FILE="/etc/llxcfg/vars.GENERATE"
VPKG_DYNAMIC_TEMPFILE="${VPKG_DIR}/tmp/vars.GENERATE"
VPKG_VALIDATE_FILE="/etc/llxcfg/vars.POSTGENERATE"
VPKG_VALIDATE_TEMPFILE="${VPKG_DIR}/tmp/vars.POSTGENERATE"
VPKG_FILE_LIST="$VPKG_STATIC $VPKG_OVERRIDE $VPKG_DYNAMIC $VPKG_VALIDATE $VPK_VARS"
VPKG_VARS_FILE="/etc/llxcfg/vars.sh"
VPKG_SCRNAMES="$VPKG_DYNAMIC $VPKG_VALIDATE"
VPKG_STATUS_DIR="${VPKG_DIR}/status"
# -----------
# functions
# -----------

vpkg_varfile_clean(){
   while [ "$1" ] ; do
      rm -f "${VPKG_STATUS_DIR}/$1"
      shift
   done
}

vpkg_varfile_unclean(){
   while [ "$1" ] ; do
      touch "${VPKG_STATUS_DIR}/$1"
      shift
   done
}

vpkg_varfile_test(){
   # return 'true' if clean
   [ -f "${VPKG_STATUS_DIR}/$1" ] && return 1
   return 0
}

vpkg_dir(){
   echo "${VPKG_INFO}/$1"
}

vpkg_reldir(){
   echo "../${VPKG_INFODIRNAME}/$1"
}

vpkg_endir(){
   pkg="$1"
   NUM_ORDER=$(printf "%03i" $(echo "$2"|sed -e "s%^0*%%"))
   if [ $NUM_ORDER -gt 999 -o $NUM_ORDER -le 0 ] ; then
      NUM_ORDER=${DEFAULT_ORDER}
   fi
   echo "${VPKG_ENABLED}/${NUM_ORDER}-${1}"
}

# llxcfg-dev     #
# -------------- #

# ---------
# variables
# ---------
VPKG_DEFAULT_NUM=200

# --------------#
# llxcfg-config
# --------------#
LCFG_ADMIN_DIR="/etc/llxcfg/config"
LCFG_PROGS_DIR="/var/lib/llxcfg/config"
LCFG_STATIC_DIR="/usr/share/llxcfg/config"
LCFG_DIRS="$LCFG_STATIC_DIR $LCFG_PROGS_DIR $LCFG_ADMIN_DIR"

# general
LLXCFG_MODE=664
LLXCFG_GROUP="llxcfg"

# -------------- #
# llxcfg-backend #
# -------------- #

LBACK_SEND="/usr/bin/llxcfg-backend-cmd"

# ------------------- #
# jumble generic cpkg #
# ------------------- #
LLXCFG_JUMBLE_SKEL="/usr/share/llxcfg/skel"
LLXCFG_JUMBLE_TEMPLATES="/usr/share/llxcfg/templates"
LLXCFG_JUMBLE_STOREDIR="/var/lib/jumble"
LLXCFG_JUMBLE_MD5SUM="$LLXCFG_JUMBLE_STOREDIR/md5sum.old"

# --------------- #
# desktops-keeper #
# --------------- #
LLXCFG_DESKTOPS_DIR="/usr/share/desktops-keeper-db"
LLXCFG_DESKTOPS_APP="llxcfg-desktops-db"
LLXCFG_DESKTOPS_ADD="llxcfg-desktops-db add"
LLXCFG_DESKTOPS_DEL="llxcfg-desktops-db del"

# -- #
# pe #
# -- #
OP_PATH="/etc/op/op.d/"
PE_PATH="/usr/share/pe"
PE_CFG_PATH="$PE_PATH/conf"
PE_ADMIN_PATH="$PE_PATH/admin"
PE_EXT_CMD="cmd"
PE_EXT_USERS="users"
PE_EXT_GROUPS="groups"
PE_CONF="pe/conf"
PE_CONF_DEFAULT="$PE_CONF/default-cmd.conf"
PE_APP_UPDATE="pe --update"
